
local LBI = LibStub:GetLibrary("LibBabble-Inventory-3.0"):GetReverseLookupTable()

local isCriteria = Overachiever.IsCriteria

local TradeSkillAch = {
  Cooking = {
    GourmetNorthrend = true,
    GourmetOutland = true
  }
}

local function TradeSkillCheck(ach, name)
  local id = OVERACHIEVER_ACHID[ach]
  local isCrit, complete = isCriteria(id, name)
  if (isCrit) then
    return id, complete
  end
end

local icons, highlights = {}
local currentButton, AchList

local function skillButtonOnEnter(self)
  currentButton = self
  local icon = icons[self]
  if (icon:IsShown()) then
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("TOPLEFT", self, "TOPRIGHT", -45, 0)
    GameTooltip:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
    GameTooltip:AddLine(OVERACHIEVER_STRINGS_REQUIREDFORMETATIP)
    GameTooltip:AddLine(" ")
    Overachiever.AddAchListToTooltip(GameTooltip, AchList[icon.index])
    GameTooltip:AddLine(" ")
    GameTooltip:Show()
    return true
  end
end

local function skillButtonOnLeave()
  currentButton = nil
  GameTooltip:Hide()
end

local function GetIcon(skillButton)
  local icon = icons[skillButton]
  if (icon) then  return icon;  end
  icon = skillButton:CreateTexture(nil, "OVERLAY")
  icon:SetTexture("Interface\\AddOns\\Overachiever\\AchShield")
  icon:SetWidth(14)
  icon:SetHeight(14)
  icon:SetPoint("LEFT", skillButton, "LEFT", 5, 0)
  icons[skillButton] = icon

  local highlight = skillButton:CreateTexture(nil, "HIGHLIGHT")
  highlight:SetTexture("Interface\\AddOns\\Overachiever\\AchShieldGlow")
  highlight:SetWidth(14)
  highlight:SetHeight(14)
  highlight:SetPoint("LEFT", skillButton, "LEFT", 5, 0)
  highlights = highlights or {}
  highlights[icon] = highlight

  local prev = skillButton:GetScript("OnEnter")
  if (prev) then
    skillButton:HookScript("OnEnter", skillButtonOnEnter)
  else
    skillButton:SetScript("OnEnter", skillButtonOnEnter)
  end
  prev = skillButton:GetScript("OnLeave")
  if (prev) then
    skillButton:HookScript("OnLeave", skillButtonOnLeave)
  else
    skillButton:SetScript("OnLeave", skillButtonOnLeave)
  end

  if (MouseIsOver(skillButton)) then
    currentButton = skillButton  -- Causes ExamineTradeSkillUI to trigger skillButtonOnEnter.
  end

  return icon
end

function Overachiever.ExamineTradeSkillUI()
  -- Hide all icons:
  for btn, icon in pairs(icons) do
    icon:Hide()
    highlights[icon]:Hide()
  end

  local ACHS = TradeSkillAch[ LBI[GetTradeSkillLine()] ]
  if (ACHS) then
    -- Find icons that should be displayed:
    local skillOffset = FauxScrollFrame_GetOffset(TradeSkillListScrollFrame)
    local skillName, skillType, id, isCrit, complete
    for i=1,TRADE_SKILLS_DISPLAYED do
      skillName, skillType = GetTradeSkillInfo(i + skillOffset)
      if (skillName and skillType ~= "header") then
        local icon
        if (AchList and type(AchList[i]) == "table") then
          wipe(AchList[i])
        end
  
        for ach in pairs(ACHS) do
          id, complete = TradeSkillCheck(ach, skillName)
          if (id) then
            if (not complete and not icon) then
              icon = GetIcon( _G["TradeSkillSkill"..i] )
              icon:Show()
              highlights[icon]:Show()
              icon.index = i
            end
            -- Remember associated achievements' IDs:
            AchList = AchList or {}
            if (AchList[i]) then
              local v = AchList[i]
              if (type(v) == "table") then
                v[#v+1] = id
              else
                AchList[i] = { v, id }
              end
            else
              AchList[i] = id
            end
          end
        end
  
      end
    end
  end

  -- Needed for when the button's contents change while the cursor is over it:
  if (currentButton and not skillButtonOnEnter(currentButton)) then
    GameTooltip:Hide()
  end
end
